(*********************************************************************
    WiegandReader.pas
     -- Access Control Card / Reader Test (Delphi 5)

    History:
    - 18.04.2004 - the first final version (ver. 1.0)
**********************************************************************)

unit WiegandReader;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons;

type
  TMainWindow = class(TForm)
    COMPortSelect: TRadioGroup;
    COM1_Select: TRadioButton;
    COM2_Select: TRadioButton;
    LEDredSButton: TSpeedButton;
    LEDgreenSButton: TSpeedButton;
    BeeperSButton: TSpeedButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    DataFormat: TLabel;
    FacilityCode: TLabel;
    CardNumber: TLabel;
    Image1: TImage;
    Label4: TLabel;
    ReaderLED: TShape;
    Label5A: TLabel;
    Label5B: TLabel;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure COM1_SelectClick(Sender: TObject);
    procedure COM2_SelectClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure LEDredSButtonClick(Sender: TObject);
    procedure LEDgreenSButtonClick(Sender: TObject);
    procedure BeeperSButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;


  TReadFrame = class(TThread)
  private
    { Private declarations }
  public
    { Public declarations }
  protected
    procedure Execute; override;
  end;


var
  MainWindow: TMainWindow;


implementation
uses PComm;                               {MOXA serial ports card library}

var  ComNo : byte;                        {active serial port number}
     TxD_buffer : array [0..15] of char;  {RS transmit buffer}
     RxD_buffer : array [0..15] of char;  {RS receive buffer}
     ReadFrameOn : boolean;               {on/off ReadFrame thread}

{$R *.DFM}
procedure TReadFrame.Execute;
{read incoming data from card reader}
var  i,j    : byte;                       {loop and buffer index}
     InChar : integer;                    {input character}

begin
  i:= 0;
  while ReadFrameOn do
    begin
      InChar:= sio_getch(ComNo);          {read one character}
      case InChar of
        $02 :  {start of frame character}
               i:= 0;
        $03 :  {end of frame character}
               begin
                 if RxD_buffer[0]='S' then
                   begin
                     {Wiegand 26 data block}
                     MainWindow.DataFormat.Caption:= 'WIEGAND 26';
                     MainWindow.FacilityCode.Caption:=
                       RxD_buffer[1]+RxD_buffer[2]+'h';
                     MainWindow.CardNumber.Caption:= '';
                     for j:= 3 to 6 do
                       MainWindow.CardNumber.Caption:=
                         MainWindow.CardNumber.Caption+RxD_buffer[j];
                     MainWindow.CardNumber.Caption:=
                       MainWindow.CardNumber.Caption+'h';
                   end
                 else if RxD_buffer[0]='L' then
                   begin
                     {Wiegand 37 data block}
                     MainWindow.DataFormat.Caption:= 'WIEGAND 37';
                     MainWindow.FacilityCode.Caption:= 'XXXX';
                     MainWindow.CardNumber.Caption:= '';
                     for j:= 1 to 9 do
                       MainWindow.CardNumber.Caption:=
                         MainWindow.CardNumber.Caption+RxD_buffer[j];
                     MainWindow.CardNumber.Caption:=
                       MainWindow.CardNumber.Caption+'h';
                   end;
               end;

        else   {other character}
          if InChar>=0 then
            begin
              RxD_buffer[i]:= Chr(InChar);  {save character in read buffer}
              i:= i+1;
            end;
      end;
    end;
end;


function OpenComPort(ComNo:byte):byte ;
{open serial port ComNo at 1200bit/sec, data format 8N1,
 receive timeout 40ms, max. char delay 4ms}
begin
  Result:= sio_open(ComNo);
  sio_ioctl(ComNo, B1200, BIT_8 or P_NONE or STOP_1);
  sio_SetReadTimeouts(ComNo, 40, 4);
end;


procedure TMainWindow.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  ReadFrameOn := false; {close ReadFrame thread}
  sio_close(ComNo);     {close active serial port}
end;


procedure TMainWindow.COM1_SelectClick(Sender: TObject);
begin
  sio_close(ComNo);                                {close COM2}
  ComNo:= 1;                                       {current serial port number}
  if (OpenComPort(ComNo)<>0) then                  {open COM1}
    begin
      ShowMessage('ERROR: Can not open COM1');
      COM1_Select.Checked:= False;
    end;
end;


procedure TMainWindow.COM2_SelectClick(Sender: TObject);
begin
  sio_close(ComNo);                                {close COM1}
  ComNo:= 2;                                       {current serial port number}
  if (OpenComPort(ComNo)<>0) then                  {open COM2}
    begin
      ShowMessage('ERROR: Can not open COM2');
      COM2_Select.Checked:= False;
    end;
end;


procedure TMainWindow.FormCreate(Sender: TObject);
begin
  ComNo:= 1;                                       {current serial port number}
  if OpenComPort(ComNo)=0 then                     {open COM1}
    COM1_Select.Checked:= True
  else
    ShowMessage('ERROR: Can not open COM1');

  ReadFrameOn := true;
  TReadFrame.Create (false);                       {start ReadFrame thread}
end;


procedure TMainWindow.LEDredSButtonClick(Sender: TObject);
{switch on/off red LED}
begin
  {send frame to the reader}
  TxD_buffer[0]:= Chr($02);                        {start of frame}
  if LEDredSButton.Down then TxD_buffer[1]:= 'S'   {set command}
                        else TxD_buffer[1]:= 'R';  {reset command}
  TxD_buffer[2]:= 'r';                             {red LED}
  TxD_buffer[3]:= Chr($03);                        {end of frame}
  sio_write(ComNo,@TxD_buffer,4);                  {send TxD_buffer}

  {set reader LED color on the screen}
  if LEDredSButton.Down then
    if LEDgreenSButton.Down then ReaderLED.Brush.Color:= clYellow
                            else ReaderLED.Brush.Color:= clRed
  else
    if LEDgreenSButton.Down then ReaderLED.Brush.Color:= clLime
                            else ReaderLED.Brush.Color:= clGray;
end;


procedure TMainWindow.LEDgreenSButtonClick(Sender: TObject);
{switch on/off green LED}
begin
  {send frame to the reader}
  TxD_buffer[0]:= Chr($02);                        {start of frame}
  if LEDgreenSButton.Down then TxD_buffer[1]:= 'S' {set command}
                          else TxD_buffer[1]:= 'R';{reset command}
  TxD_buffer[2]:= 'g';                             {green LED}
  TxD_buffer[3]:= Chr($03);                        {end of frame}
  sio_write(ComNo,@TxD_buffer,4);                  {send TxD_buffer}

  {set reader LED color on the screen}
  if LEDgreenSButton.Down then
    if LEDredSButton.Down then ReaderLED.Brush.Color:= clYellow
                          else ReaderLED.Brush.Color:= clLime
  else
    if LEDredSButton.Down then ReaderLED.Brush.Color:= clRed
                          else ReaderLED.Brush.Color:= clGray;
end;


procedure TMainWindow.BeeperSButtonClick(Sender: TObject);
{switch on/off beeper}
begin
  {send frame to the reader}
  TxD_buffer[0]:= Chr($02);                        {start of frame}
  if BeeperSButton.Down then TxD_buffer[1]:= 'S'   {set command}
                        else TxD_buffer[1]:= 'R';  {reset command}
  TxD_buffer[2]:= 'b';                             {beeper LED}
  TxD_buffer[3]:= Chr($03);                        {end of frame}
  sio_write(ComNo,@TxD_buffer,4);                  {send TxD_buffer}
end;

end.


